@@ -166,7 +166,7 @@ int ykp_configure_command(YKP_CONFIG *cfg, uint8_t command)
 	case SLOT_CONFIG:
 		break;
 	case SLOT_CONFIG2:
-		if (!cfg->yk_major_version >= 2) {
+		if (!(cfg->yk_major_version >= 2)) {
 			ykp_errno = YKP_EOLDYUBIKEY;
 			return 0;
 		}
@@ -187,17 +187,32 @@ int ykp_configure_command(YKP_CONFIG *cfg, uint8_t command)
 		}
 		break;
 	case SLOT_DEVICE_CONFIG:
+		if(!(cfg->yk_major_version <= 5)) {
+			ykp_errno = YKP_EYUBIKEYVER;
+			return 0;
+		} /* we have an intentional fall-through to the next case here */
 	case SLOT_SCAN_MAP:
-	case SLOT_NDEF2:
 		if(!(cfg->yk_major_version >= 3)) {
 			ykp_errno = YKP_EYUBIKEYVER;
 			return 0;
 		}
 		break;
+	case SLOT_YK4_SET_DEVICE_INFO:
+		if(!(cfg->yk_major_version >= 5)) {
+			ykp_errno = YKP_EYUBIKEYVER;
+			return 0;
+		}
+		break;
+	case SLOT_NDEF2:
+		if(cfg->yk_major_version != 3 && cfg->yk_major_version != 5) {
+			ykp_errno = YKP_EYUBIKEYVER;
+			return 0;
+		}
+		break;
 	case SLOT_NDEF:
 		/* NDEF is available for neo, thus within 2.1 from build 4 */
 		if (!((cfg->yk_major_version == 2 && cfg->yk_minor_version == 1 &&
-			  cfg->yk_build_version >= 4) || cfg->yk_major_version == 3)) {
+			  cfg->yk_build_version >= 4) || cfg->yk_major_version == 3 || cfg->yk_major_version >= 5)) {
 			ykp_errno = YKP_EYUBIKEYVER;
 			return 0;
 		}
@@ -975,7 +990,7 @@ static int _ykp_legacy_export_config(const YKP_CONFIG *cfg, char *buf, size_t le
 
 		/* uid: */
 		if (key_bits_in_uid) {
-			strncpy(buffer, "n/a", 3);
+			strncpy(buffer, "n/a", 4);
 		} else {
 			yubikey_hex_encode(buffer, (const char *)ycfg.uid, UID_SIZE);
 		}
@@ -1068,8 +1083,9 @@ int ykp_import_config(YKP_CONFIG *cfg, const char *buf, size_t len,
 		return _ykp_json_import_cfg(cfg, buf, len);
 	} else if(format == YKP_FORMAT_LEGACY) {
 		ykp_errno = YKP_ENOTYETIMPL;
+	} else {
+		ykp_errno = YKP_EINVAL;
 	}
-	ykp_errno = YKP_EINVAL;
 	return 0;
 }
 int ykp_write_config(const YKP_CONFIG *cfg,
